@@ -17,15 +17,15 @@
 
 def BoolProperty(name="", description="", default=False):
     pass
-def BoolVectorProperty(name="", description="", default=(False, False, False)):
+def BoolVectorProperty(name="", description="", default=(False, False, False), size=3):
     pass
 def IntProperty(name="", description="", default=0):
     pass
-def IntVectorProperty(name="", description="", default=(0, 0, 0)):
+def IntVectorProperty(name="", description="", default=(0, 0, 0), size=3):
     pass
 def FloatProperty(name="", description="", default=0):
     pass
-def FloatVectorProperty(name="", description="", default=(0, 0, 0)):
+def FloatVectorProperty(name="", description="", default=(0, 0, 0), size=3):
     pass
 def StringProperty(name="", description="", default=""):
     pass
